var httpProxy = require('http-proxy');
var util = require('util');

// Custom Winston logger
var winston = require('winston');
winston.remove(winston.transports.Console)
    .add(winston.transports.Console, {
        level: "silly",
        colorize: true
    });

exports.logger = winston;
var helpers = {

// This function wraps the response object so that we can intercept
// writes.
    responseWrap: function (res, host, port) {
        var write = res.write;
        res.write = function(data) {
            winston.info("Response data intercepted from "+host+":"+port+" : ");
            data.toString().split("\n").forEach(function(line) {
                winston.verbose("data: "+line);
            });
            write.apply(this, arguments);
        }
        return res;
    },

    requestWrap: function (req, host, port) {
        winston.info("Request intercepted from "+host+":"+port+" : ");
        util.inspect({
            url: req.url,
            headers: req.headers
        }).split("\n").forEach(function (line) {
            winston.verbose("data: "+line);
        });
        return req;        
    }
};

// This wraps httpProxy.
var instrumented = module.exports = (function (httpProxy) {
    var createServer = httpProxy.createServer;

    httpProxy.createServer = function (cb) {
        return createServer.call(this, function (req, res, proxy) {
            var host = req.socket.remoteAddress;
            var port = req.socket.remotePort;

            helpers.requestWrap(req, host, port);

            // We need to wrap the proxy.proxyRequest method.
            var proxyRequest = proxy.proxyRequest;

            proxy.proxyRequest = function (req, res, options) {
                proxyRequest.call(this, req, helpers.responseWrap(res, host, port), options);
            }

            cb(req, res, proxy);
        });

    }

    httpProxy.helpers = helpers;

    return httpProxy;
})(httpProxy);
